@@ -15,7 +15,7 @@
 
 There should be a file 'ftesting.zcml' in the current directory.
 
-$Id: functional.py 90031 2008-08-20 17:07:35Z fdrake $
+$Id: functional.py 90098 2008-08-21 21:10:21Z fdrake $
 """
 import logging
 import os.path
@@ -38,6 +38,7 @@ from zope.publisher.publish import publish
 from zope.security.interfaces import Forbidden, Unauthorized
 from zope.testing import doctest
 
+import zope.app.appsetup.product
 import zope.app.testing.setup
 from zope.app.appsetup.appsetup import multi_database
 from zope.app.debug import Debugger
@@ -202,6 +203,7 @@ class FunctionalTestSetup(object):
                 BaseDatabaseFactory(name, self._base_storages)
                 for name in database_names
                 )[0][0]
+            self.dbstack = []
             self.app = Debugger(self.db, config_file)
 
             self.connection = None
@@ -248,33 +250,44 @@ class FunctionalTestSetup(object):
     base_storage = property(_get_base_storage, _set_base_storage)
 
     def _close_databases(self):
+        # This is really careful to unregister the databases before attempting
+        # to close anything.  Zope Corporation has a couple of large
+        # multi-database applications that get bitten if we're not careful
+        # like this, but we've not been able to write a concise test case yet.
         base = component.getGlobalSiteManager()
-        for name, db in self.db.databases.iteritems():
+        dbs = []
+        for name, db in list(component.getUtilitiesFor(IDatabase)):
+            ok = base.unregisterUtility(db, IDatabase, name)
+            assert ok
+            dbs.append(db)
+        if self.connection:
+            self.connection.close()
+            self.connection = None
+        for db in dbs:
             db.close()
-            base.unregisterUtility(db, IDatabase, name)
 
     def setUp(self):
         """Prepares for a functional test case."""
         # Tear down the old demo storages (if any) and create fresh ones
         abort()
-        self._close_databases()
+        self.dbstack.append((self.db, self.connection))
+        self.connection = None
         self.db = self.app.db = multi_database(
             DerivedDatabaseFactory(name, self._base_storages)
             for name in self._database_names
             )[0][0]
-        self.connection = None
 
     def tearDown(self):
         """Cleans up after a functional test case."""
         abort()
-        if self.connection:
-            self.connection.close()
-            self.connection = None
         self._close_databases()
+        self.db, self.connection = self.dbstack.pop()
         setSite(None)
 
     def tearDownCompletely(self):
         """Cleans up the setup done by the constructor."""
+        self._close_databases()
+        assert self.dbstack == []
         zope.app.testing.setup.placefulTearDown()
         self._config_file = False
         self._product_config = None
