@@ -13,7 +13,7 @@
 ##############################################################################
 """Test tcpdoc
 
-$Id: tests.py 90032 2008-08-20 17:09:30Z fdrake $
+$Id: tests.py 90098 2008-08-21 21:10:21Z fdrake $
 """
 import os
 import re
@@ -413,16 +413,38 @@ def doctest_FunctionalTestSetup_clears_global_utilities():
     See https://bugs.launchpad.net/zope3/+bug/251273
 
         >>> setup = FunctionalTestSetup(ftesting_zcml)
+
+    At this point, there are registrations for the base databases created by
+    the initialization:
+
+        >>> base, = getAllUtilitiesRegisteredFor(IDatabase)
+
+    Setting up for a test causes overriding registrations to be made:
+
         >>> setup.setUp()
+        >>> dbs = list(getAllUtilitiesRegisteredFor(IDatabase))
+        >>> len(dbs)
+        2
+        >>> base in dbs
+        True
+        >>> dbs.remove(base)
+        >>> override, = dbs
+
+    Tearing down the test context causes the overriding database to be
+    removed:
+
         >>> setup.tearDown()
+        >>> list(getAllUtilitiesRegisteredFor(IDatabase)) == [base]
+        True
 
-        >>> len(getAllUtilitiesRegisteredFor(IDatabase))
-        0
-
-    Clean up:
+    Tearing down the fixture causes the base database registration to be
+    removed:
 
         >>> setup.tearDownCompletely()
 
+        >>> list(getAllUtilitiesRegisteredFor(IDatabase))
+        []
+
     """
 
 
